"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const router_method_factory_1 = require("../helpers/router-method-factory");
const Router = require("router");
const queryString = require("query-string");
const root = require("window-or-global");
class BrowserAdapter {
    constructor() {
        this.routerMethodFactory = new router_method_factory_1.RouterMethodFactory();
        this._router = Router();
    }
    use(...args) {
        return this._router.use(...args);
    }
    get(path, ...args) {
        return this._router.get(path.substring(1), ...args);
    }
    post(path, ...args) {
        return this._router.post(path.substring(1), ...args);
    }
    head(path, ...args) {
        return this._router.head(path.substring(1), ...args);
    }
    delete(path, ...args) {
        return this._router.delete(path.substring(1), ...args);
    }
    put(path, ...args) {
        return this._router.put(path.substring(1), ...args);
    }
    patch(path, ...args) {
        return this._router.patch(path.substring(1), ...args);
    }
    options(path, ...args) {
        return this._router.options(path.substring(1), ...args);
    }
    listen(port, hostname, callback) {
        let window;
        if (!process['browser']) {
            window = {
                history: {
                    push: () => { },
                    pushState: () => { },
                },
                location: {
                    href: '/'
                }
            };
        }
        else {
            window = root;
        }
        this.handleUrlChange(window.location.href);
        const pushState = window.history.pushState;
        window.history.pushState = () => {
            pushState.apply(window.history, arguments);
            this.handleUrlChange(window.location.href);
        };
    }
    handleUrlChange(url = '/') {
        const req = {
            query: queryString.parseUrl(url).query,
            url,
            method: 'GET',
        };
        this._router(req, {}, error => error ? console.error(error) : null);
    }
    reply(response, body, statusCode) {
    }
    render(response, view, options) {
    }
    setErrorHandler(handler) {
        return this.use(handler);
    }
    setNotFoundHandler(handler) {
        return this.use(handler);
    }
    setHeader(response, name, value) {
        return response.set(name, value);
    }
    getHttpServer() {
        return null;
    }
    setHttpServer(httpServer) {
    }
    getInstance() {
        return this._router;
    }
    close() {
        return this._router.close();
    }
    set(...args) {
        return this._router.set(...args);
    }
    enable(...args) {
        return this._router.enable(...args);
    }
    disable(...args) {
        return this._router.disable(...args);
    }
    engine(...args) {
        return this._router.engine(...args);
    }
    useStaticAssets(path, options) {
        return this;
    }
    setBaseViewsDir(path) {
        return this.set('views', path);
    }
    setViewEngine(engine) {
        return this.set('view engine', engine);
    }
    getRequestMethod(request) {
        return request.method;
    }
    getRequestUrl(request) {
        return request.url;
    }
    createMiddlewareFactory(requestMethod) {
        return this.routerMethodFactory
            .get(this._router, requestMethod)
            .bind(this._router);
    }
}
exports.BrowserAdapter = BrowserAdapter;
